@@ -19,6 +19,10 @@ struct ScreenInfo
   SizeID current_size;
   short current_rate;
   Rotation current_rotation;
+
+  SizeID old_size;
+  short old_rate;
+  Rotation old_rotation;
   
   XRRScreenConfiguration *config;
   XRRScreenSize *sizes;
@@ -73,6 +77,40 @@ read_display_info (GdkDisplay *display)
   return info;
 }
 
+void
+update_display_info (struct DisplayInfo *info, GdkDisplay *display)
+{
+  struct ScreenInfo *screen_info;
+  GdkScreen *screen;
+  GdkWindow *root_window;
+  int i;
+
+  g_assert (info->n_screens == gdk_display_get_n_screens (display));
+  
+  for (i = 0; i < info->n_screens; i++)
+    {
+      screen = gdk_display_get_screen (display, i);
+      
+      screen_info = &info->screens[i];
+
+      screen_info->old_rate = screen_info->current_rate;
+      screen_info->old_size = screen_info->current_size;
+      screen_info->old_rotation = screen_info->current_rotation;
+      
+      screen_info->current_width = gdk_screen_get_width (screen);
+      screen_info->current_height = gdk_screen_get_height (screen);
+
+      root_window = gdk_screen_get_root_window (screen);
+      XRRFreeScreenConfigInfo (screen_info->config);
+      screen_info->config = XRRGetScreenInfo (gdk_x11_display_get_xdisplay (display),
+					      gdk_x11_drawable_get_xid (GDK_DRAWABLE (root_window)));
+
+      screen_info->current_rate = XRRConfigCurrentRate (screen_info->config);
+      screen_info->current_size = XRRConfigCurrentConfiguration (screen_info->config, &screen_info->current_rotation);
+      screen_info->sizes = XRRConfigSizes (screen_info->config, &screen_info->n_sizes);
+    }
+}
+
 static int
 get_current_resolution (struct ScreenInfo *screen_info)
 {
@@ -150,6 +188,8 @@ apply_config (struct DisplayInfo *info)
 					      GDK_CURRENT_TIME);
 	}
     }
+
+  update_display_info (info, display);
   
   /* xscreensaver should handle this itself, but does not currently so we hack
    * it.  Ignore failures in case xscreensaver is not installed */
@@ -182,14 +222,28 @@ revert_config (struct DisplayInfo *info)
       status = XRRSetScreenConfigAndRate (xdisplay, 
 					  screen_info->config,
 					  gdk_x11_drawable_get_xid (GDK_DRAWABLE (root_window)),
-					  screen_info->current_size,
-					  screen_info->current_rotation,
-					  screen_info->current_rate,
+					  screen_info->old_size,
+					  screen_info->old_rotation,
+					  screen_info->old_rate,
 					  GDK_CURRENT_TIME);
-
-		generate_resolution_menu(screen_info);
-		generate_rate_menu(screen_info);
+      
     }
+
+  update_display_info (info, display);
+
+  /* Need to update the menus to the new settings */
+  for (i = 0; i < info->n_screens; i++)
+    {
+      struct ScreenInfo *screen_info = &info->screens[i];
+      
+      generate_resolution_menu (screen_info);
+      generate_rate_menu (screen_info);
+    }
+
+  /* xscreensaver should handle this itself, but does not currently so we hack
+   * it.  Ignore failures in case xscreensaver is not installed */
+  g_spawn_command_line_async ("xscreensaver-command -restart", NULL);
+  
   return 0;
 }
 
@@ -201,7 +255,7 @@ wrap_in_label (GtkWidget *child, char *text)
   char *str;
 
   vbox = gtk_vbox_new (FALSE, 6);
-  label = 0;
+  label = NULL;
 
   label = gtk_label_new ("");
 
@@ -569,7 +623,8 @@ run_revert_dialog (struct DisplayInfo *info,
   gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 12);
   gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-  gtk_window_set_title (GTK_WINDOW (dialog), "");
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Keep Resolution"));
+  gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_CENTER_ALWAYS);
   
   label = gtk_label_new (NULL);
   str = g_strdup_printf ("<b>%s</b>", _("Do you want to keep this resolution?"));
@@ -703,8 +758,10 @@ cb_dialog_response (GtkDialog *dialog, gint response_id, struct DisplayInfo *inf
 	  
       if (apply_config (info))
 	{
+	  gtk_widget_hide(GTK_WIDGET(dialog));
 	  if (!run_revert_dialog (info, GTK_WIDGET (dialog)))
 	    {
+	      gtk_widget_show(GTK_WIDGET(dialog));
 	      revert_config (info);
 	      return;
 	    }
@@ -746,7 +803,7 @@ main (int argc, char *argv[])
       XRRQueryVersion (xdisplay, &major, &minor) == 0)
     {
       GtkWidget *msg_dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-						   _("The XServer does not support the XRandR extension.  Runtime resolution changes to the display size are not available."));
+						   _("The X Server does not support the XRandR extension.  Runtime resolution changes to the display size are not available."));
       gtk_dialog_run (GTK_DIALOG (msg_dialog));
       gtk_widget_destroy (msg_dialog);
       exit (0);
@@ -763,7 +820,7 @@ main (int argc, char *argv[])
   info = read_display_info (display);
   dialog = create_dialog (info);
 
-  gtk_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (cb_dialog_response), info);
+  g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (cb_dialog_response), info);
   gtk_widget_show (dialog);
 
   gtk_main ();
