@@ -4,7 +4,6 @@
 #include <config.h>
 
 #include <string.h>
-#include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
@@ -62,6 +61,8 @@ static gboolean reload_themes;
 static gboolean initial_meta_theme_set = FALSE;
 static GdkPixbuf *default_image = NULL;
 static GdkPixbuf *broken_image = NULL;
+static gboolean themes_loaded = FALSE;
+static gboolean reverted = FALSE;
 
 static GnomeThemeMetaInfo custom_meta_theme_info;
 static GnomeThemeMetaInfo initial_meta_theme_info;
@@ -534,7 +535,13 @@ meta_theme_selection_changed (GtkTreeSelection *selection,
   GtkTreeIter iter;
   gchar *meta_theme_name;
   GtkTreeModel *model;
-
+  gchar *current_gtk_theme;
+  gchar *current_window_theme;
+  gchar *current_icon_theme;
+  GConfClient *client;
+  GnomeWindowManager *window_manager;
+  GnomeWMSettings wm_settings;
+  
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
       gtk_tree_model_get (model, &iter,
@@ -561,8 +568,35 @@ meta_theme_selection_changed (GtkTreeSelection *selection,
   if (setting_model)
     return;
 
-  if (meta_theme_info)
+  if (meta_theme_info) 
     gnome_meta_theme_set (meta_theme_info);
+  
+  if (!themes_loaded) {
+    client = gconf_client_get_default ();
+
+    /* Get the settings */
+    current_gtk_theme = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
+    current_icon_theme = gconf_client_get_string (client, ICON_THEME_KEY, NULL);
+    window_manager = gnome_wm_manager_get_current (gdk_display_get_default_screen (gdk_display_get_default ()));
+    wm_settings.flags = GNOME_WM_SETTING_THEME;
+    if (window_manager) {
+      gnome_window_manager_get_settings (window_manager, &wm_settings);
+      current_window_theme = g_strdup (wm_settings.theme);
+    } else
+      current_window_theme = g_strdup ("");
+
+	initial_meta_theme_info.name = g_strdup ("__Initial Theme__");
+	initial_meta_theme_info.gtk_theme_name = g_strdup (current_gtk_theme);
+	initial_meta_theme_info.metacity_theme_name = g_strdup (current_window_theme);
+ 	initial_meta_theme_info.icon_theme_name = g_strdup (current_icon_theme);
+	themes_loaded = TRUE;
+  } else {
+	  if (!reverted) {
+        gtk_widget_set_sensitive(WID("meta_theme_revert_button"),TRUE);
+	  } else {
+		reverted = FALSE;
+	  }
+  }
 }
 
 /* This function will adjust the list to reflect the current theme
@@ -885,7 +919,19 @@ static void
 window_settings_changed (GnomeWindowManager *window_manager,
 			 GladeXML           *dialog)
 {
-  /* There is no way to know what changed, so we just check it all */
+  static gchar *window_theme = NULL;
+  GnomeWMSettings wm_settings;
+
+  wm_settings.flags = GNOME_WM_SETTING_THEME;
+  gnome_window_manager_get_settings (window_manager, &wm_settings);
+  if (window_theme == NULL || strcmp (wm_settings.theme, window_theme) != 0)
+    {
+      g_free (window_theme);
+      window_theme = g_strdup (wm_settings.theme);
+    }
+  else
+    return;
+
   update_settings_from_gconf ();
   gnome_theme_details_update_from_gconf ();
 }
@@ -1036,7 +1082,7 @@ static void
 gnome_meta_theme_installer_run_cb (GtkWidget *button,
 				   GtkWidget *parent_window)
 {
-  gnome_theme_installer_run (parent_window, NULL, FALSE);
+  gnome_theme_installer_run (parent_window, NULL);
 }
 
 
@@ -1080,8 +1126,20 @@ apply_font_clicked (GtkWidget *button,
       gconf_client_set_string (client, FONT_KEY, meta_theme_info->application_font, NULL);
     }  
 }
-	    
-	    
+
+static void
+revert_theme_clicked (GtkWidget *button,
+						gpointer data)
+{
+  GladeXML *dialog;
+
+  gnome_meta_theme_set(&initial_meta_theme_info);
+  
+  dialog = gnome_theme_manager_get_theme_dialog ();
+  gtk_widget_set_sensitive(WID("meta_theme_revert_button"), FALSE);  
+  reverted = TRUE;
+}
+	
 static void
 setup_dialog (GladeXML *dialog)
 {
@@ -1107,14 +1165,19 @@ setup_dialog (GladeXML *dialog)
   gtk_size_group_add_widget (size_group, WID ("meta_theme_background1_button"));
   gtk_size_group_add_widget (size_group, WID ("meta_theme_font2_button"));
   gtk_size_group_add_widget (size_group, WID ("meta_theme_background2_button"));
+  gtk_size_group_add_widget (size_group, WID ("meta_theme_revert_button"));
   g_object_unref (size_group);
 
+  gtk_widget_set_sensitive(WID("meta_theme_revert_button"),FALSE);
+
   g_signal_connect (G_OBJECT (WID ("meta_theme_install_button")), "clicked", G_CALLBACK (gnome_meta_theme_installer_run_cb), parent);
 
   g_signal_connect (G_OBJECT (WID ("meta_theme_details_button")), "clicked", gnome_theme_details_show, NULL);
 
   g_signal_connect (G_OBJECT (WID ("meta_theme_font1_button")), "clicked", G_CALLBACK (apply_font_clicked), dialog);
 
+  g_signal_connect (G_OBJECT (WID ("meta_theme_revert_button")), "clicked", G_CALLBACK (revert_theme_clicked),NULL);
+
   setup_meta_tree_view (GTK_TREE_VIEW (WID ("meta_theme_treeview")),
 			(GCallback) meta_theme_selection_changed,
 			dialog);
@@ -1163,7 +1226,7 @@ setup_dialog (GladeXML *dialog)
   g_signal_connect (G_OBJECT (parent), "drag-leave", G_CALLBACK (gnome_theme_manager_drag_leave_cb), NULL);
   g_signal_connect (G_OBJECT (parent), "drag-data-received",G_CALLBACK (gnome_theme_manager_drag_data_received_cb), NULL);
 
-  capplet_set_icon (parent, "gnome-ccthemes.png");
+  capplet_set_icon (parent, "gnome-settings-theme");
 
   update_font_button_state (dialog);
   gtk_widget_show (parent);
@@ -1337,7 +1400,7 @@ gnome_theme_manager_drag_data_received_cb (GtkWidget *widget, GdkDragContext *co
 		gnome_vfs_uri_list_unref (uris);
 	}
 
-	gnome_theme_installer_run (widget, filename, FALSE);
+	gnome_theme_installer_run (widget, filename);
 	g_free (filename);
 }
 
