@@ -280,6 +280,9 @@ set_server_from_gconf (GConfEntry *ignored)
 			XkbAccessXFeedbackMask	|
 			XkbControlsEnabledMask,
 			desc);
+
+	XkbFreeKeyboard (desc, XkbAllComponentsMask, True);
+
 	XSync (GDK_DISPLAY (), FALSE);
 	gdk_error_trap_pop ();
 }
@@ -366,7 +369,8 @@ warning_dialog_post (GtkWidget *dialog, GtkWidget **hbox, GCallback response,
 		     gboolean *enabled, gchar *title, gchar *user_action_string, gchar *query)
 {
 	GtkWidget *label;
-	gchar *label_markup_string = g_strconcat ("<big>", query, "</big>\n\n", user_action_string, NULL);
+	gchar *label_markup_string = g_strconcat ("<span weight=\"bold\" size=\"larger\">", 
+						  query, "</span>\n\n", user_action_string, NULL);
 
 	if (!dialog)
 	{
@@ -524,6 +528,8 @@ set_gconf_from_server (GConfEntry *ignored)
 		}
 	}
 
+	XkbFreeKeyboard (desc, XkbAllComponentsMask, True);
+
 	changed |= (stickykeys_changed | slowkeys_changed);
 
 	if (changed) {
